import ssl
import certifi
import aiohttp

import logging

import asyncio
from . import loggers

logging.basicConfig(level=logging.INFO)

class From_user:
    def __init__(self, fullname: str, user_id: int):
        self.full_name = fullname
        self.id = user_id
    
class Chat:
    def __init__(self, id: int):
        self.id = id
    
class Reply_to_message:
    def __init__(self, full_name: str, user_id: int, message_id: int):
        self.full_name = full_name
        self.user_id = user_id
        self.message_id = message_id
        self.from_user = From_user(fullname=full_name, user_id=user_id)

class MessageObject:
    def __init__(self, chat_id: int, message_id: int, fullname: str, user_id: int,
                 reply_to_message_fullname: str, reply_to_message_user_id: int, reply_to_message_message_id: int,
                 token: str, type_chat: str, message_text: str):
        
        self.message_id = message_id
        self.text = message_text
        self.type = type_chat
        self.reply_to_message = Reply_to_message(full_name=reply_to_message_fullname, user_id=reply_to_message_user_id, message_id=reply_to_message_message_id)
        self.from_user = From_user(fullname=fullname, user_id=user_id)
        self.chat = Chat(chat_id)
        self.__token = token
        self.__url = "https://api.telegram.org/bot" + self.__token + "/"
        
        self.session = None
        self.ssl_context = ssl.create_default_context(cafile=certifi.where())
    
    async def start_session(self):
        if not self.session:
            self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=self.ssl_context))

    async def close_session(self):
        if self.session:
            await self.session.close()
            self.session = None
    
    async def answer(self, message: str, parse_mode: str="HTML", reply_markup=None):
        await self.start_session()
        
        payload = {
            "chat_id": self.chat.id,
            "text": message,
            "parse_mode": parse_mode
        }
        
        if reply_markup:
            payload["reply_markup"] = reply_markup
        
        try:
            async with self.session.post(self.__url + "sendMessage", json=payload) as response:
                response.raise_for_status()  # Бросает исключение для HTTP-ошибок
                data = await response.json()
                loggers.types.info("The message has been sent successfully.")
                return data.get("result")
        except Exception as e:
            loggers.types.error(f"{e}")
            return {"ok": False, "error": str(e)}
        finally:
            await self.close_session()
    
    async def reply(self, message: str, parse_mode: str="HTML", reply_markup=None):
        await self.start_session()
        
        payload = {
            "chat_id": self.chat.id,
            "text": message,
            "reply_to_message_id": self.message_id,
            "parse_mode": parse_mode
        }

        if reply_markup:
            payload["reply_markup"] = reply_markup

        try:
            async with self.session.post(self.__url + "sendMessage", json=payload) as response:
                response.raise_for_status()
                data = await response.json()
                loggers.types.info("The message has been sent successfully.")
                return data.get("result")
        except Exception as e:
            loggers.types.error(f"{e}")
            return {"ok": False, "error": str(e)}
        finally:
            await self.close_session()

class CallbackQuery:
    def __init__(self, obj: dict, token: str):
        self.id = obj.get("id", None)
        self.message_json = obj.get("message", {})
        self.message = MessageObject(
    message_id=self.message_json.get("message_id", None),
    chat_id=self.message_json.get("chat", {}).get("id", None),
    fullname=self.message_json.get("from", {}).get("first_name", None),
    user_id=self.message_json.get("from", {}).get("id", None),
    reply_to_message_fullname=self.message_json.get("reply_to_message", {}).get("from", {}).get("first_name", None),
    reply_to_message_user_id=self.message_json.get("reply_to_message", {}).get("from", {}).get("id", None),
    reply_to_message_message_id=self.message_json.get("reply_to_message", {}).get("message_id", None),
    token=token,
    type_chat=self.message_json.get("chat", {}).get("type", None),
    message_text=self.message_json.get("text", None)
)
        # self.message = MessageObject(message_id=self.message.get("message_id", None), chat_id=obj.get("message", None).get("chat", None).get("id", None), fullname=self.message.get("from", None).get("first_name", None), user_id=self.message.get("from", None).get("id", None), reply_to_message_fullname=self.message.get("reply_to_message", None).get("from", None).get("first_name", None), reply_to_message_user_id=self.message.get("reply_to_message", None).get("from", None).get("id", None), reply_to_message_message_id=self.message.get("reply_to_message", None).get("message_id", None), token=obj.get("token", None), type_chat=self.message.get("chat", None).get("type", None), message_text=self.message.get("text", None))
        self.chat_instance = obj.get("chat_instance", None)
        self.data = obj.get("data", None)

class Permissions:
    def __init__(self, obj: dict):
        can_send_messages: bool = obj.get("can_send_messages", None)
        can_send_media_messages: bool = obj.get("can_send_media_messages", None)
        can_send_audios: bool = obj.get("can_send_audios", None)
        can_send_documents: bool = obj.get("can_send_documents", None)
        can_send_photos: bool = obj.get("can_send_photos", None)
        can_send_videos: bool = obj.get("can_send_videos", None)
        can_send_video_notes: bool = obj.get("can_send_video_notes", None)
        can_send_voice_notes: bool = obj.get("can_send_voice_notes", None)
        can_send_polls: bool = obj.get("can_send_polls", None)
        can_send_other_messages: bool = obj.get("can_send_other_messages", None)
        can_add_web_page_previews: bool = obj.get("can_add_web_page_previews", None)
        can_change_info: bool = obj.get("can_change_info", None)
        can_invite_users: bool = obj.get("can_invite_users", None)
        can_pin_messages: bool = obj.get("can_pin_messages", None)
        can_manage_topics: bool = obj.get("can_manage_topics", None)

class GetChat:
    def __init__(self, obj: dict):
        self.id: int = obj.get("id", None)
        self.title: str = obj.get("title", None)
        self.type: str = obj.get("type", None)
        self.invite_link: str = obj.get("invite_link", None)
        self.permissions = Permissions(obj.get("permissions", None))
        self.join_to_send_messages: bool = obj.get("join_to_send_messages", None)
        self.max_reaction_count: int = obj.get("max_reaction_count", None)
        self.accent_color_id: int = obj.get("accent_color_id", None)

class CommandObject:
    def __init__(self, text_message: str):
        self.args = " ".join(text_message.split()[1:])
        self.args_list = text_message.split()[1:]

class GetMe:
    def __init__(self, obj: dict):
        self.id: int = obj.get("id", None)
        self.is_bot: bool = obj.get("is_bot", None)
        self.first_name: str = obj.get("first_name", None)
        self.last_name: str = obj.get("last_name", None)
        self.username: str = obj.get("username", None)
        self.language_code: str = obj.get("language_code", None)
        self.can_join_groups: bool = obj.get("can_join_groups", None)
        self.can_read_all_group_messages: bool = obj.get("can_read_all_group_messages", None)
        self.supports_inline_queries: bool = obj.get("supports_inline_queries", None)



from aiotele.types import MessageObject, GetChat, CommandObject, GetMe, CallbackQuery
import aiohttp
import asyncio

from aiotele import loggers

import ssl
import certifi

import inspect

from typing import List

class CommandHandler:
    def __init__(self, token: str):
        self.commands = {}
        self.__token = token
        self.default_handler = None  # Хендлер для любого сообщения

    def command(self, command: str = None, commands: List[str] = None, prefix: str = None):
        def wrapper(func):
            if command is None and commands is None:
                self.default_handler = func  # Устанавливаем обработчик для любого текста
            elif command is None and commands:
                for _command in commands:
                    if prefix != None:
                        for i in prefix:
                            _command = i + _command
                            self.commands[_command] = func
            elif isinstance(command, str):
                if prefix != None:
                    for i in prefix:
                        _command = i + command
                        self.commands[_command] = func
                else:
                    _command = command
                    self.commands[_command] = func
            return func
        return wrapper

    async def handle(self, update, bot):
        message = update.get("message", {})
        text = message.get("text", "")
        command = text.split(" ")[0]

        # Проверяем, есть ли обработчик для команды или общий обработчик для любого сообщения
        handler = self.commands.get(command, self.default_handler)

        if handler:
            from_info = message.get("from", {})
            user_id = from_info.get("id", 0)
            full_name = f"{from_info.get('first_name', '')} {from_info.get('last_name', '')}".strip()
            chat = message.get("chat", {})
            type_chat = chat.get("type", "None")
            chat_id = chat.get("id", 0)
            message_id = message.get("message_id", 0)

            reply_to_info = message.get("reply_to_message", {})
            reply_to_message_id = reply_to_info.get("message_id", None)
            reply_to_from_info = reply_to_info.get("from", {})
            reply_to_user_id = reply_to_from_info.get("id", None)
            reply_to_full_name = f"{reply_to_from_info.get('first_name', '')} {reply_to_from_info.get('last_name', '')}".strip()

            msg_obj = MessageObject(
                fullname=full_name,
                user_id=user_id,
                chat_id=chat_id,
                message_id=message_id,
                reply_to_message_fullname=reply_to_full_name,
                reply_to_message_user_id=reply_to_user_id,
                reply_to_message_message_id=reply_to_message_id,
                token=self.__token,
                type_chat=type_chat,
                message_text=text
            )
            command_obj = CommandObject(
                text_message=text,
            )
            sig = inspect.signature(handler)
            params = len(sig.parameters)
            if params == 1:
                await handler(msg_obj)
            else:
                await handler(msg_obj, command_obj)

class CallbackDataHandler:
    def __init__(self, token: str):
        self.commands = {}
        self.__token = token
        self.default_handler = None  # Хендлер для любого сообщения

    def callback(self, command: str = None):
        def wrapper(func):
            if command is None:
                self.default_handler = func  # Устанавливаем обработчик для любого текста
            elif command is None:
                _command = i + _command
                self.commands[_command] = func
            elif isinstance(command, str):
                self.commands[command] = func
            return func
        return wrapper

    async def handle(self, update, bot):
        message = update.get("message", {})
        callback = update.get("callback_query", None)
        command = callback.get("data", None)

        # Проверяем, есть ли обработчик для команды или общий обработчик для любого сообщения
        handler = self.commands.get(command, self.default_handler)

        if handler:
            callback_obj = CallbackQuery(callback, self.__token)
            await handler(callback_obj)

class Bot:
    def __init__(self, TOKEN: str):
        self.token = TOKEN
        self.url = f"https://api.telegram.org/bot{self.token}/"
        self.__message_handler = CommandHandler(token=self.token)
        self.__callback_handler = CallbackDataHandler(token=self.token)
        self.update_offset = 0
        self.session = None
        self.ssl_context = ssl.create_default_context(cafile=certifi.where())
    
    def message_handler(self, command: str = None, commands: List[str] = None, prefix: str = None):
        return self.__message_handler.command(command, commands, prefix)

    def callback_handler(self, command: str = None):
        return self.__callback_handler.callback(command)

    async def start_session(self):
        if not self.session:
            self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=self.ssl_context))

    async def close_session(self):
        if self.session:
            await self.session.close()
            self.session = None

    async def send_message(self, chat_id: int, message: str, parse_mode: str="HTML", reply_markup=None):
        await self.start_session()
        
        payload = {
            "chat_id": chat_id,
            "text": message,
            "parse_mode": parse_mode
        }
        
        if reply_markup:
            payload["reply_markup"] = reply_markup

        try:
            async with self.session.post(self.url + "sendMessage", json=payload) as response:
                response.raise_for_status()  # Бросает исключение для HTTP-ошибок
                data = await response.json()
                loggers.bot.info("The message has been sent successfully.")
                return data.get("result")
        except Exception as e:
            loggers.bot.error(f"{e}")
            return {"ok": False, "error": str(e)}
    
    async def send_photo(self, chat_id: int, file_path: str, caption: str = None, parse_mode: str="HTML", reply_markup=None):
        await self.start_session()
        
        try:
            # Чтение файла и создание FormData
            with open(file_path, "rb") as photo_file:
                form_data = aiohttp.FormData()
                form_data.add_field("chat_id", str(chat_id))
                form_data.add_field("photo", photo_file, filename=file_path.split("/")[-1])
                form_data.add_field("parse_mode", parse_mode)

                if caption:
                    form_data.add_field("caption", caption)

                if reply_markup:
                    form_data.add_field("reply_markup", reply_markup)
                
                # Отправка запроса
                async with self.session.post(f"{self.url}sendPhoto", data=form_data) as response:
                    response.raise_for_status()  # Бросает исключение при HTTP-ошибке
                    loggers.bot.info("The message has been sent successfully.")
                    data = await response.json()
                    return data.get("result")
        
        except aiohttp.ClientError as e:
            loggers.bot.error(f"{e}")
            return {"ok": False, "error": str(e)}
        
        except FileNotFoundError:
            error_message = f"File not found: {file_path}"
            loggers.bot.error(error_message)
            return {"ok": False, "error": error_message}
    
    async def get_chat(self, chat_id: int):
        await self.start_session()
        
        try:
            payload = {
                "chat_id": chat_id
            }
            # Отправка запроса
            async with self.session.post(f"{self.url}getChat", json=payload) as response:
                response.raise_for_status()  # Бросает исключение при HTTP-ошибке
                loggers.bot.info("The message has been sent successfully.")
                data = await response.json()
                return GetChat(data.get("result"))
        except aiohttp.ClientError as e:
            loggers.bot.error(f"{e}")
            return {"ok": False, "error": str(e)}
    
    async def get_updates(self):
        await self.start_session()
        async with self.session.get(self.url + "getUpdates", params={"offset": self.update_offset}) as response:
            if response.status == 200:
                data = await response.json()
                for update in data.get("result", []):
                    if update.get("callback_query", None) != None:
                        await self.__callback_handler.handle(update, self)
                        self.update_offset = update["update_id"] + 1
                        break
                    self.update_offset = update["update_id"] + 1
                    await self.__message_handler.handle(update, self)
            else:
                loggers.bot.error(await response.text())

    async def get_me(self):
        await self.start_session()
        async with self.session.get(self.url + "getMe") as response:
            if response.status == 200:
                data = await response.json()
                return GetMe(data.get("result"))
            else:
                loggers.bot.error(await response.text())
                return {"ok": False, "error": await response.text()}
    
    async def edit_text(self, chat_id: int, message_id: int, text: str):
        await self.start_session()
        payload = {
            "chat_id": chat_id,
            "message_id": message_id,
            "text": text
        }
        
        async with self.session.post(self.url + "editMessageText", json=payload) as response:
            if response.status == 200:
                data = await response.json()
                return data.get("result")
            else:
                loggers.bot.error(await response.text())
                return {"ok": False, "error": await response.text()}
    
    async def delete_message(self, chat_id: int, message_id: int):
        await self.start_session()
        payload = {
            "chat_id": chat_id,
            "message_id": message_id,
        }
        
        async with self.session.post(self.url + "deleteMessage", json=payload) as response:
            if response.status == 200:
                data = await response.json()
                return data.get("result")
            else:
                loggers.bot.error(await response.text())
                return {"ok": False, "error": await response.text()}
    
    async def delete_webhook(self, drop_pending_updates: bool = False):
        await self.start_session()
        payload = {
            "drop_pending_updates": str(drop_pending_updates).lower()
        }
        
        url = f"{self.url}deleteWebhook"
        async with self.session.post(url, params=payload) as response:
            if response.status == 200:
                data = await response.json()
                return data.get("result")
            else:
                loggers.bot.error(await response.text())
                return {"ok": False, "error": await response.text()}

    async def run(self):
        bot = await self.get_me()
        loggers.bot.info("Poll started")
        loggers.bot.info(f"Bot with the name '{bot.first_name}' and the username @{bot.username} has been launched")
        try:
            while True:
                await self.get_updates()
                await asyncio.sleep(1)
        except asyncio.CancelledError:
            pass
        finally:
            loggers.bot.info(f"Poll stopped")
            await self.close_session()
